'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.Backup = void 0;
const fs = require("fs");
const libcheck_1 = require("./libcheck");
const lib = new libcheck_1.LibCheck();
class Backup {
    alert;
    /**
     * A class to create backups for your database.
     * Recomended to use this in different file.
     * @param {Boolean} alerts alert things to the console?
     */
    constructor(alerts) {
        alerts != undefined
            ? this.alert = alerts
            : this.alert = false;
    }
    /**
     * create backup file or folder
     * @param {String} adapter directory or .json file. If file -> type .json at the end.
     */
    createBackup(adapter) {
        if (adapter.startsWith('system/')) {
            const fileError = '\u001b[1;31mYou can\'t edit system folder.\u001b[0m';
            throw new Error(fileError);
        }
        switch (adapter.endsWith('.json')) {
            case true: {
                adapter = adapter.replace(/\//i, '').replace(/\./i, '');
                lib.checkAny(adapter);
                let backupfile = fs.readFileSync(adapter, 'utf-8');
                fs.writeFileSync(__dirname + '/backup/' + 'unsorted/' + adapter.split('/')[adapter.split('/').length - 1], backupfile);
                if (this.alert == true)
                    console.log('Alert: created a new backup file: /unsorted/' + adapter + '\n');
                break;
            }
            case false: {
                adapter = adapter.replace(/\//i, '').replace(/\./i, '');
                lib.checkDir(adapter);
                let readDir = fs.readdirSync(adapter);
                readDir.forEach((file) => {
                    let fl = fs.readFileSync(adapter + '/' + file, 'utf-8');
                    if (!fs.existsSync(__dirname + '/backup/' + adapter)) {
                        fs.mkdirSync(__dirname + '/backup/' + adapter);
                    }
                    fs.writeFileSync(__dirname + '/backup/' + adapter + '/' + file, fl);
                });
                if (this.alert == true)
                    console.log('Alert: created a new backup dir: /' + adapter + '/\n');
                break;
            }
        }
    }
    /**
     * remove backup file or folder
     * @param {String} adapter directory or .json file. If file -> type .json at the end.
     */
    removeBackup(adapter) {
        if (adapter.startsWith('system/')) {
            const fileError = '\u001b[1;31mYou can\'t edit system folder.\u001b[0m';
            throw new Error(fileError);
        }
        switch (adapter.endsWith('.json')) {
            case true: {
                lib.checkAny(__dirname + '/backup/unsorted/' + adapter);
                fs.unlinkSync(__dirname + '/backup/' + 'unsorted/' + adapter);
                if (this.alert == true)
                    console.log('Alert: deleted backup file: /unsorted/' + adapter + '\n');
                break;
            }
            case false: {
                lib.checkAny(__dirname + '/backup/' + adapter);
                let readDir = fs.readdirSync(__dirname + '/backup/' + adapter);
                readDir.forEach((file) => {
                    fs.unlinkSync(__dirname + '/backup/' + adapter + '/' + file);
                });
                fs.rmdirSync(__dirname + '/backup/' + adapter);
                if (this.alert == true)
                    console.log('Alert: removed backup dir: /' + adapter + '/ with all files in it.\n');
                break;
            }
        }
    }
    /**
     * restore backup files
     * @param {String} adapter directory or .json file. If file -> type .json at the end.
     * @param {String|null} directory a directory where to restore.
     */
    restoreBackup(adapter, directory) {
        if (adapter.startsWith('system/')) {
            const fileError = '\u001b[1;31mYou can\'t edit system folder.\u001b[0m';
            throw new Error(fileError);
        }
        var lastdir = directory;
        lastdir == null
            ? lastdir = process.cwd()
            : process.cwd() + '/' + directory;
        lib.checkDir(lastdir);
        switch (adapter.endsWith('.json')) {
            case true: {
                if (adapter.split(/[\\\/]+/).length == 2) {
                    // single file from sorted: /adapterDir/adapterFile.json
                    let spladapter = adapter.split(/[\\\/]+/);
                    let file = fs.readFileSync(__dirname + '/backup/' + spladapter[0] + '/' + spladapter[1], 'utf8');
                    fs.writeFileSync(lastdir + '/' + spladapter[1], file);
                }
                else {
                    // single file from /unsorted/
                    let file = fs.readFileSync(__dirname + '/backup/unsorted/' + adapter, 'utf8');
                    fs.writeFileSync(lastdir + '/' + adapter, file);
                }
                break;
            }
            case false: {
                // entire /dir/ for each file
                let readDir = fs.readdirSync(__dirname + '/backup/' + adapter);
                readDir.forEach((table) => {
                    let file = fs.readFileSync(__dirname + '/backup/' + adapter + '/' + table, 'utf8');
                    fs.writeFileSync(lastdir + '/' + table, file);
                });
                break;
            }
        }
    }
    /**
     * view list of files to backup
     * @param {Boolean} logs log table in console?
     * @param {Boolean} returns return object?
     */
    viewBackup(logs, returns) {
        if (logs == false && returns == false)
            return;
        const backup = fs.readdirSync(__dirname + '/backup', 'utf-8');
        var table = {};
        var lengths = 0;
        backup.forEach(dir => {
            if (String(dir).length > lengths)
                lengths = String(dir).length;
            let readdir = fs.readdirSync(__dirname + '/backup/' + dir, 'utf-8');
            let arr = [];
            readdir.forEach((backed) => {
                arr.push(backed);
                if (backed.length > lengths)
                    lengths = backed.length;
            });
            table[dir] = arr;
        });
        if (logs == true) {
            var visualTable = '';
            visualTable += '\n|' + (' ' + '-'.repeat(lengths) + ' |').repeat(Object.keys(table).length) + '\n|';
            for (const [key] of Object.entries(table)) {
                visualTable += ' ' + key + ' '.repeat(lengths - key.length) + ' |';
            }
            visualTable += '\n|' + (' ' + '-'.repeat(lengths) + ' |').repeat(Object.keys(table).length) + '\n|';
            var maxlines = 0;
            for (const [key] of Object.entries(table)) {
                if (table[key].length > maxlines)
                    maxlines = table[key].length;
            }
            for (let line = 0; line < maxlines; line++) {
                var i = -1;
                for (const [key] of Object.entries(table)) {
                    i++;
                    if (table[key].length > maxlines)
                        maxlines = table[key].length;
                    let arr = [];
                    table[key].forEach((dir) => {
                        arr.push(dir);
                    });
                    visualTable += (' ' + String(arr[line] != undefined ? arr[line] : ' ') + ' '.repeat(lengths - String(arr[line] != undefined ? arr[line] : ' ').length) + ' |');
                }
                visualTable += '\n|';
            }
            visualTable += (' ' + '-'.repeat(lengths) + ' |').repeat(Object.keys(table).length) + '\n';
            console.log(visualTable);
        }
        if (returns == true)
            return table;
    }
}
exports.Backup = Backup;
