export declare class Database {
    tablePath: string;
    settings: boolean | undefined;
    /**
     * The main class of database app.
     * @param {String} TablePath A path, that uses your database. (Supports multiple databases)
     * @arg {Boolean} Alerts - Alert writes and removes to the console.
     */
    constructor(TablePath: string, Alerts?: boolean);
    /**
     * Remove table inside foolder.
     * @param {String} tableName Name of the table (/tablePath/tableName.json).
     */
    removeTable(tableName: string): void;
    /**
     * Create a new table inside foolder.
     * @param {String} tableName Name of the table (/tablePath/tableName.json).
     * @param {Object} strict Parameters for strict requirements.
     */
    createTable(tableName: string, strict?: object): void;
    /**
     * Read whole amount of increments.
     * Also returns aLL increment values.
     */
    readIncrement(): string | void;
    /**
     * Check increment file.
     * Recomended to always use this function when you run code (especially for big projects with nodemon)
     * if you run this on a server and it creates this error, it will automatically create a new increment file from saved backup.
     */
    checkIncrement(): void;
    /**
     * edit increment value.
     * @param {String} tableName Name of the table (/tablePath/tableName.json).
     * @param {Number} value
     */
    editIncrement(tableName: string, value: any): void;
    /**
     * Write a line in database.
     * @param {String} tableName Name of the table (/tablePath/tableName.json).
     * @param {String} key Key in a table to write and then use it as beacon.
     * @param {Any} value
     */
    write(tableName: string, key: string, value: any): void;
    /**
     * Check if this key exists.
     * @returns {Boolean} Boolean
     * @param {String} tableName Name of the table (/tablePath/tableName.json).
     * @param {String} key Key in a table to write and then use it as beacon.
     */
    check(tableName: string, key: string): boolean;
    /**
     * Read and return data of this key in database.
     * @returns {Any} If this key doesnt exist it returns undefined.
     * @param {String} tableName Name of the table (/tablePath/tableName.json).
     * @param {String} key Key in a table to write and then use it as beacon.
     */
    read(tableName: string, key: string): any;
    /**
     * Edit single value of this key in database.
     * @param {String} tableName Name of the table (/tablePath/tableName.json).
     * @param {String} key Key in a table to write and then use it as beacon.
     * @param {Any} value Can work as subkey for subvalue
     * @param {Any} subvalue New value of this subkey.
     */
    edit(tableName: string, key: string, value: any, subvalue?: any): void;
    /**
     * Remove a line in database.
     * @param {String} tableName Name of the table (/tablePath/tableName.json).
     * @param {String} key Key in a table to write and then use it as beacon.
     */
    remove(tableName: string, key: string): void;
}
