'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.Database = void 0;
const fs = require("fs");
const libcheck_1 = require("./libcheck");
const lib = new libcheck_1.LibCheck();
class Database {
    tablePath;
    settings;
    /**
     * The main class of database app.
     * @param {String} TablePath A path, that uses your database. (Supports multiple databases)
     * @arg {Boolean} Alerts - Alert writes and removes to the console.
     */
    constructor(TablePath, Alerts) {
        this.tablePath = TablePath;
        Alerts
            ? this.settings = Alerts
            : this.settings = false;
        lib.checkDir(this.tablePath);
    }
    /**
     * Remove table inside foolder.
     * @param {String} tableName Name of the table (/tablePath/tableName.json).
     */
    removeTable(tableName) {
        lib.checkDir(this.tablePath);
        if (fs.existsSync(this.tablePath + '/' + tableName + '.json')) {
            fs.unlink(this.tablePath + '/' + tableName + '.json', function (error) {
                if (error)
                    throw error;
            });
            if (fs.existsSync(this.tablePath + '/' + tableName + '-rules.json')) {
                fs.unlink(this.tablePath + '/' + tableName + '-rules.json', function (error) {
                    if (error)
                        throw error;
                });
            }
            if (this.settings && this.settings == true)
                console.log('Alert: deleted table with name "' + tableName + '".');
        }
        else {
            const fileError = '\u001b[1;31mUnable to find table ./' + tableName + '.json\u001b[0m';
            throw new Error(fileError);
        }
    }
    /**
     * Create a new table inside foolder.
     * @param {String} tableName Name of the table (/tablePath/tableName.json).
     * @param {Object} strict Parameters for strict requirements.
     */
    createTable(tableName, strict) {
        lib.checkDir(this.tablePath);
        switch (strict) {
            default: {
                if (fs.existsSync(this.tablePath + '/' + tableName + '.json')) {
                    if (this.settings && this.settings == true)
                        console.log('Alert: created a new table "' + tableName + '" with strict mode:\n' + JSON.stringify(strict, null, '\t'));
                    let result = {};
                    fs.writeFileSync(this.tablePath + '/' + tableName + '.json', JSON.stringify(result));
                    fs.writeFileSync(this.tablePath + '/' + tableName + '-rules.json', JSON.stringify(strict, null, '\t'));
                }
                else {
                    if (this.settings && this.settings == true)
                        console.log('Alert: created a new table "' + tableName + '" with strict mode. Cleared old one.');
                    let result = {};
                    fs.writeFileSync(this.tablePath + '/' + tableName + '.json', JSON.stringify(result));
                    fs.writeFileSync(this.tablePath + '/' + tableName + '-rules.json', JSON.stringify(strict, null, '\t'));
                }
                break;
            }
            case undefined: {
                if (fs.existsSync(this.tablePath + '/' + tableName + '.json')) {
                    if (this.settings && this.settings == true)
                        console.log('Alert: created a new table "' + tableName + '" withour strict mode.');
                    let result = {};
                    fs.writeFileSync(this.tablePath + '/' + tableName + '.json', JSON.stringify(result));
                }
                else {
                    if (this.settings && this.settings == true)
                        console.log('Alert: created a new table "' + tableName + '" without strict mode. Cleared old one.');
                    let result = {};
                    fs.writeFileSync(this.tablePath + '/' + tableName + '.json', JSON.stringify(result));
                }
                break;
            }
        }
    }
    /**
     * Read whole amount of increments.
     * Also returns aLL increment values.
     */
    readIncrement() {
        lib.checkDir(this.tablePath);
        let incr = fs.readFileSync(this.tablePath + '/increment.json', 'utf-8');
        incr = JSON.parse(incr);
        console.log(JSON.stringify(incr, null, '\t'));
        return incr;
    }
    /**
     * Check increment file.
     * Recomended to always use this function when you run code (especially for big projects with nodemon)
     * if you run this on a server and it creates this error, it will automatically create a new increment file from saved backup.
     */
    checkIncrement() {
        if (!fs.existsSync(this.tablePath + '/increment.json')) {
            const backup = fs.readFileSync(__dirname + '/backup/increment.json', 'utf-8');
            let an = JSON.parse(backup);
            fs.writeFileSync(this.tablePath + '/increment.json', JSON.stringify(an, null, '\t'));
        }
    }
    /**
     * edit increment value.
     * @param {String} tableName Name of the table (/tablePath/tableName.json).
     * @param {Number} value
     */
    editIncrement(tableName, value) {
        lib.checkDir(this.tablePath);
        let incr = fs.readFileSync(this.tablePath + '/increment.json', 'utf-8');
        incr = JSON.parse(incr);
        if (value < 1) {
            const valueError = '\u001b[1;31m"value" must be greater than 0.\u001b[0m';
            throw new Error(valueError);
        }
        var ex = false;
        var doub = false;
        for (const [key, val] of Object.entries(incr)) {
            if (key == tableName)
                ex = true;
            if (val == value)
                doub = true;
        }
        switch (ex) {
            case true: {
                if (doub == true) {
                    if (this.settings && this.settings == true)
                        console.log('\nAlert: Increment value of ' + tableName + ' == value. Nothing to change.');
                }
                else {
                    if (this.settings && this.settings == true)
                        console.log('\nAlert: Increment value of ' + tableName + ' is now ' + value + '.');
                    incr[tableName] = value;
                    fs.writeFileSync(this.tablePath + '/increment.json', JSON.stringify(incr, null, '\t'));
                    fs.writeFileSync(__dirname + '/backup/increment.json', JSON.stringify(incr, null, '\t'));
                }
                break;
            }
            case false: {
                const listError = '\u001b[1;31mcouldn\'t find.';
                throw new Error(listError);
            }
        }
    }
    /**
     * Write a line in database.
     * @param {String} tableName Name of the table (/tablePath/tableName.json).
     * @param {String} key Key in a table to write and then use it as beacon.
     * @param {Any} value
     */
    write(tableName, key, value) {
        lib.checkDir(this.tablePath);
        lib.checkFile(this.tablePath, tableName);
        var rules = null;
        let data = fs.readFileSync(this.tablePath + '/' + tableName + '.json', 'utf-8');
        if (data && tableName.endsWith('-rules')) {
            const fileError = '\u001b[1;31mDon\'t edit RULE file: ./' + tableName + '.json\u001b[0m';
            throw new Error(fileError);
        }
        if (fs.existsSync(this.tablePath + '/' + tableName + '-rules.json')) {
            rules = fs.readFileSync(this.tablePath + '/' + tableName + '-rules.json', 'utf-8');
        }
        let incr = fs.readFileSync(this.tablePath + '/increment.json', 'utf-8');
        data = JSON.parse(data);
        data[key] = value;
        if (rules != null) {
            if (value instanceof Object) {
                rules = JSON.parse(rules);
                var isValue = true;
                for (const [input] of Object.entries(value)) {
                    for (const [arule] of Object.entries(rules)) {
                        if (input === arule) {
                            isValue = true;
                        }
                        else {
                            isValue = false;
                        }
                    }
                }
                if (isValue == false) {
                    const validError = '\u001b[1;31mparameters in "value" are not the same as in rule file.\u001b[0m';
                    throw new Error(validError);
                }
                for (const [rule, rulevalue] of Object.entries(rules)) {
                    if (this.settings && this.settings == true)
                        console.log(`${rule}: ${rulevalue}   =>   ${(typeof value[rule]).toUpperCase()} (${(typeof value[rule]).toUpperCase() == rulevalue ? true : false})`);
                    switch (typeof value[rule]) {
                        case "string": {
                            if (rulevalue == 'AUTO_INCREMENT' && value[rule] == "AUTO_INCREMENT") {
                                console.log('# "' + rule + '" is now AUTO_INCREMENT.');
                                break;
                            }
                            if (rulevalue != 'STRING') {
                                const typeError = '\u001b[1;31min rule-file "' + value[rule] + '" is instance of ' + rulevalue + '\u001b[0m';
                                throw new Error(typeError);
                            }
                            break;
                        }
                        case "number": {
                            if (rulevalue == 'AUTO_INCREMENT' && value[rule] == "AUTO_INCREMENT") {
                                console.log('# "' + rule + '" is now AUTO_INCREMENT.');
                                break;
                            }
                            if (rulevalue != 'NUMBER') {
                                const typeError = '\u001b[1;31min rule-file "' + value[rule] + '" is instance of ' + rulevalue + '\u001b[0m';
                                throw new Error(typeError);
                            }
                            break;
                        }
                        case "object": {
                            if (rulevalue == 'AUTO_INCREMENT' && value[rule] == "AUTO_INCREMENT") {
                                console.log('# "' + rule + '" is now AUTO_INCREMENT.');
                                break;
                            }
                            if (Array.isArray(value[rule])) {
                                if (rulevalue == 'AUTO_INCREMENT' && value[rule] == "AUTO_INCREMENT") {
                                    console.log('# "' + rule + '" is now AUTO_INCREMENT.');
                                    break;
                                }
                                if (rulevalue != 'ARRAY') {
                                    const typeError = '\u001b[1;31min rule-file "' + value[rule] + '" is instance of ' + rulevalue + '\u001b[0m';
                                    throw new Error(typeError);
                                }
                            }
                            else {
                                if (rulevalue != 'OBJECT') {
                                    const typeError = '\u001b[1;31min rule-file "' + value[rule] + '" is instance of ' + rulevalue + '\u001b[0m';
                                    throw new Error(typeError);
                                }
                            }
                            break;
                        }
                        case "boolean": {
                            if (rulevalue == 'AUTO_INCREMENT' && value[rule] == "AUTO_INCREMENT") {
                                console.log('# "' + rule + '" is now AUTO_INCREMENT.');
                                break;
                            }
                            if (rulevalue != 'BOOLEAN') {
                                const typeError = 'in rule-file "' + value[rule] + '" is instance of ' + rulevalue + '\u001b[0m';
                                throw new Error(typeError);
                            }
                            break;
                        }
                    }
                }
            }
            else {
                const typeError = '"value" must be an OBJECT in strict mode';
                throw new Error(typeError);
            }
        }
        if (data[key] instanceof Object) {
            for (const [subkey, subvalue] of Object.entries(data[key])) {
                if (subvalue == 'AUTO_INCREMENT') {
                    incr = JSON.parse(incr);
                    !incr[tableName]
                        ? incr[tableName] = 1
                        : incr[tableName] = incr[tableName] + 1;
                    fs.writeFileSync(this.tablePath + '/increment.json', JSON.stringify(incr, null, '\t'));
                    fs.writeFileSync(__dirname + '/backup/increment.json', JSON.stringify(incr, null, '\t'));
                    data[key][subkey] = Number(data[key][subkey].replace('AUTO_INCREMENT', incr[tableName]));
                }
            }
        }
        else {
            if (data[key] == 'AUTO_INCREMENT') {
                incr = JSON.parse(incr);
                !incr[tableName]
                    ? incr[tableName] = 1
                    : incr[tableName] = incr[tableName] + 1;
                fs.writeFileSync(this.tablePath + '/increment.json', JSON.stringify(incr, null, '\t'));
                fs.writeFileSync(__dirname + '/backup/increment.json', JSON.stringify(incr, null, '\t'));
                data[key] = Number(data[key].replace('AUTO_INCREMENT', incr[tableName]));
            }
        }
        fs.writeFileSync(this.tablePath + '/' + tableName + '.json', JSON.stringify(data, null, '\t'));
        if (this.settings && this.settings == true)
            console.log('\nAlert: Added a new line to /' + this.tablePath + '/' + tableName + '.json:\n' + key + ':\t' + JSON.stringify(data[key], null, '\t'));
    }
    /**
     * Check if this key exists.
     * @returns {Boolean} Boolean
     * @param {String} tableName Name of the table (/tablePath/tableName.json).
     * @param {String} key Key in a table to write and then use it as beacon.
     */
    check(tableName, key) {
        lib.checkDir(this.tablePath);
        lib.checkFile(this.tablePath, tableName);
        let data = fs.readFileSync(this.tablePath + '/' + tableName + '.json', 'utf-8');
        return (JSON.parse(data)[key] == undefined
            ? false
            : true);
    }
    /**
     * Read and return data of this key in database.
     * @returns {Any} If this key doesnt exist it returns undefined.
     * @param {String} tableName Name of the table (/tablePath/tableName.json).
     * @param {String} key Key in a table to write and then use it as beacon.
     */
    read(tableName, key) {
        lib.checkDir(this.tablePath);
        lib.checkFile(this.tablePath, tableName);
        let data = fs.readFileSync(this.tablePath + '/' + tableName + '.json', 'utf-8');
        if (data && tableName.endsWith('-rules')) {
            const fileError = '\u001b[1;31mDon\'t edit RULE file: ./' + tableName + '.json' + '\u001b[0m';
            throw new Error(fileError);
        }
        if (!JSON.parse(data)[key] || JSON.parse(data)[key] == undefined) {
            return undefined;
        }
        return JSON.parse(data)[key];
    }
    /**
     * Edit single value of this key in database.
     * @param {String} tableName Name of the table (/tablePath/tableName.json).
     * @param {String} key Key in a table to write and then use it as beacon.
     * @param {Any} value Can work as subkey for subvalue
     * @param {Any} subvalue New value of this subkey.
     */
    edit(tableName, key, value, subvalue) {
        lib.checkDir(this.tablePath);
        lib.checkFile(this.tablePath, tableName);
        var rules = null;
        let data = fs.readFileSync(this.tablePath + '/' + tableName + '.json', 'utf-8');
        if (data && tableName.endsWith('-rules')) {
            const fileError = '\u001b[1;31mDon\'t edit RULE file: ./' + tableName + '.json' + '\u001b[0m';
            throw new Error(fileError);
        }
        if (fs.existsSync(this.tablePath + '/' + tableName + '-rules.json')) {
            rules = fs.readFileSync(this.tablePath + '/' + tableName + '-rules.json', 'utf-8');
        }
        data = JSON.parse(data);
        if (!data[key]) {
            const keyError = 'can\'t find "' + key + '" key in database. Unable to change it.' + '\u001b[0m';
            throw new Error(keyError);
        }
        if (rules != null) {
            if (subvalue) {
                rules = JSON.parse(rules);
                for (const [rule, rulevalue] of Object.entries(rules)) {
                    if (rule === value) {
                        console.log(`${rule}: ${rulevalue}   =>   ${(typeof value).toUpperCase()} (${rulevalue === (typeof value).toUpperCase() ? true : false})`);
                        if (subvalue == "AUTO_INCREMENT") {
                            const valueError = '\u001b[1;31mdon\'t use "' + subvalue + '" value in this function. \u001b[1;36mIf its already stated as AUTO_INCREMENT, why do you need to change it?' + '\u001b[0m';
                            throw new Error(valueError);
                        }
                        switch (typeof subvalue) {
                            case "string": {
                                if (rulevalue != 'STRING') {
                                    const typeError = '\u001b[1;31min rule-file "' + value[rule] + '" is instance of ' + rulevalue + '\u001b[0m';
                                    throw new Error(typeError);
                                }
                                break;
                            }
                            case "number": {
                                if (rulevalue != 'NUMBER') {
                                    const typeError = '\u001b[1;31min rule-file "' + value[rule] + '" is instance of ' + rulevalue + '\u001b[0m';
                                    throw new Error(typeError);
                                }
                                break;
                            }
                            case "object": {
                                if (Array.isArray(value[rule])) {
                                    if (rulevalue != 'ARRAY') {
                                        const typeError = '\u001b[1;31min rule-file "' + value[rule] + '" is instance of ' + rulevalue + '\u001b[0m';
                                        throw new Error(typeError);
                                    }
                                }
                                else {
                                    if (rulevalue != 'OBJECT') {
                                        const typeError = '\u001b[1;31min rule-file "' + value[rule] + '" is instance of ' + rulevalue + '\u001b[0m';
                                        throw new Error(typeError);
                                    }
                                }
                                break;
                            }
                            case "boolean": {
                                if (rulevalue != 'BOOLEAN') {
                                    const typeError = '\u001b[1;31min rule-file "' + value[rule] + '" is instance of ' + rulevalue + '\u001b[0m';
                                    throw new Error(typeError);
                                }
                                break;
                            }
                        }
                    }
                }
            }
            else {
                const argsError = '\u001b[1;31mcan\'t edit whole table in strict mode. Use subvalue instead.' + '\u001b[0m';
                throw new Error(argsError);
            }
        }
        if (data[key] instanceof Object) {
            if (!String(subvalue)) {
                data[key] = value;
            }
            else {
                data[key][value] = subvalue;
            }
            fs.writeFileSync(this.tablePath + '/' + tableName + '.json', JSON.stringify(data, null, '\t'));
        }
        else {
            data[key] = value;
            fs.writeFileSync(this.tablePath + '/' + tableName + '.json', JSON.stringify(data, null, '\t'));
        }
        if (this.settings && this.settings == true)
            console.log('\nAlert: Edited a line in /' + this.tablePath + '/' + tableName + '.json:\n' + key + ':\t' + JSON.stringify(data[key], null, '\t'));
    }
    /**
     * Remove a line in database.
     * @param {String} tableName Name of the table (/tablePath/tableName.json).
     * @param {String} key Key in a table to write and then use it as beacon.
     */
    remove(tableName, key) {
        lib.checkDir(this.tablePath);
        lib.checkFile(this.tablePath, tableName);
        let data = fs.readFileSync(this.tablePath + '/' + tableName + '.json', 'utf-8');
        if (data && tableName.endsWith('-rules')) {
            const fileError = '\u001b[1;31mDon\'t edit RULE file: ./' + tableName + '.json' + '\u001b[0m';
            throw new Error(fileError);
        }
        if (!JSON.parse(data)[key]) {
            const keyError = '\u001b[1;31mUnable to find key: "' + key + '"' + '\u001b[0m';
            throw new Error(keyError);
        }
        data = JSON.parse(data);
        const oldResult = JSON.stringify(data[key], null, '\t');
        data[key] = undefined;
        fs.writeFileSync(this.tablePath + '/' + tableName + '.json', JSON.stringify(data, null, '\t'));
        if (this.settings && this.settings == true)
            console.log('\nAlert: Removed a line from /' + this.tablePath + '/' + tableName + '.json:\n' + key + ':\t' + oldResult + '\nWARNING: THIS IS REMOVED LINE!');
    }
}
exports.Database = Database;
