'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.Tools = void 0;
class Tools {
    /**
     * A random token generator.
     * @returns {String} String
     * @param {Number} length Length of the token you want to create
     * @param {String} alphabet An alphabet of allowed symbols in generated token
     * @param {String} alphabet "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789" by default
     * @param {Array} nonallowed Remove some type(-s) of characters
     * @param {Array} nonallowed ["upperCaseString", "lowerCaseString", "number", "other?"]
     * @param {Array} nonallowed "other" is used only if argument alphabet exists
     * @param {Array} nonallowed empty array by default
     */
    generateToken(length, alphabet, nonallowed) {
        switch (true) {
            case (length < 2): {
                var lengthError = '\u001b[1;31mparameter "length" must be greater than 1.' + '\u001b[0m';
                throw new Error(lengthError);
            }
            case (length > 100): {
                var lengthError = '\u001b[1;31mparameter "length" must be less than 101.' + '\u001b[0m';
                throw new Error(lengthError);
            }
        }
        var result = '';
        var alphabetDefault = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
        var characters = '';
        var charactersLength = 0;
        switch (!alphabet && !nonallowed) {
            case true: {
                characters = alphabetDefault;
                charactersLength = characters.length;
                for (var i = 0; i < length; i++) {
                    result += characters.charAt(Math.floor(Math.random() * charactersLength));
                }
                return result;
            }
            case false: {
                if (alphabet) {
                    switch (true) {
                        case (alphabet.length < 2): {
                            var lengthError = '\u001b[1;31m"alphabet"\'s length must be greater than 1 symbol.' + '\u001b[0m';
                            throw new Error(lengthError);
                        }
                    }
                    characters = alphabet;
                }
                if (nonallowed) {
                    characters = alphabet ? alphabet : alphabetDefault;
                    var upperCase = "";
                    var lowerCase = "";
                    var numbers = "";
                    var others = "";
                    for (let i = 0; i < characters.length; i++) {
                        if (characters[i].match(/[0-9]/)) {
                            numbers += characters[i];
                        }
                        if (characters[i].match(/[a-z]/)) {
                            lowerCase += characters[i];
                        }
                        if (characters[i].match(/[A-Z]/)) {
                            upperCase += characters[i];
                        }
                        if (!characters[i].match(/[0-9]/) && !characters[i].match(/[A-Z]/) && !characters[i].match(/[a-z]/) && alphabet) {
                            others += characters[i];
                        }
                    }
                    if (nonallowed.every(elem => ['lowerCaseString', 'upperCaseString', 'number', alphabet ? 'other' : undefined].includes(elem))) {
                        if ((nonallowed).indexOf("lowerCaseString") >= 0) {
                            characters = characters.replace(lowerCase, '');
                        }
                        if ((nonallowed).indexOf("upperCaseString") >= 0) {
                            characters = characters.replace(upperCase, '');
                        }
                        if ((nonallowed).indexOf("number") >= 0) {
                            characters = characters.replace(numbers, '');
                        }
                        if ((nonallowed).indexOf("other") >= 0 && alphabet) {
                            characters = characters.replace(others, '');
                        }
                    }
                    else {
                        const arrayError = alphabet
                            ? '\u001b[1;31m"notallowed" can only read these params: ["lowerCaseString", "upperCaseString", "number", "other"]' + '\u001b[0m'
                            : '\u001b[1;31m"notallowed" can only read these params: ["lowerCaseString", "upperCaseString", "number"]' + '\u001b[0m';
                        throw new Error(arrayError);
                    }
                }
                charactersLength = characters.length;
                for (var i = 0; i < length; i++) {
                    result += characters.charAt(Math.floor(Math.random() * charactersLength));
                }
                return result;
            }
        }
    }
    /**
     * Generates string list type table.
     * @param {Object} table object of your table, where every element should be string[] | number[] | boolean[]
     * @param {Number|'longest'} length how wide each column has to be
     */
    generateTable(table, length) {
        var lengths = 0;
        var minlength = 0;
        for (const [key, value] of Object.entries(table)) {
            if (String(key).length > minlength)
                minlength = String(key).length;
            value.forEach((element) => {
                if (String(element).length > minlength)
                    minlength = String(element).length;
            });
        }
        if (length == 'longest') {
            lengths = minlength;
        }
        else {
            if (minlength < length) {
                const lengthError = '\u001b[1;31mLength must be greater or equal to minimum length\u001b[0m\n' + `\t\u001b[0;30mmin: \u001b[0m\u001b[0;33m${minlength}\u001b[0m \u001b[1;31m<\u001b[0m \u001b[1;36m${length}\u001b[0m` + '\u001b[0m';
                throw new Error(lengthError);
            }
            else
                lengths = length;
        }
        if (lengths < 1) {
            const lengthError = '\u001b[1;31mLength must be greater than 0' + '\u001b[0m';
            throw new Error(lengthError);
        }
        var visualTable = '';
        visualTable += '|' + (' ' + '-'.repeat(lengths) + ' |').repeat(Object.keys(table).length) + '\n|';
        for (const [key] of Object.entries(table)) {
            visualTable += ' ' + key + ' '.repeat(lengths - key.length) + ' |';
        }
        visualTable += '\n|' + (' ' + '-'.repeat(lengths) + ' |').repeat(Object.keys(table).length) + '\n|';
        var maxlines = 0;
        for (const [key] of Object.entries(table)) {
            if (table[key].length > maxlines)
                maxlines = table[key].length;
        }
        for (let line = 0; line < maxlines; line++) {
            let i = -1;
            for (const [key] of Object.entries(table)) {
                i++;
                if (table[key].length > maxlines)
                    maxlines = table[key].length;
                let arr = [];
                table[key].forEach((dir) => {
                    arr.push(dir);
                });
                visualTable += (' ' + String(arr[line] != undefined ? arr[line] : ' ') + ' '.repeat(lengths - String(arr[line] != undefined ? arr[line] : ' ').length) + ' |');
            }
            visualTable += '\n|';
        }
        visualTable += (' ' + '-'.repeat(lengths) + ' |').repeat(Object.keys(table).length) + '\n';
        return visualTable;
    }
}
exports.Tools = Tools;
